@@ -38,6 +38,7 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 #include "json.h"
 
 #ifdef JSON_NAMESPACE
+#ifdef __cplusplus
 # define SOAP_TYPE__boolean             SOAP_TYPE_json__boolean
 # define SOAP_TYPE__i4                  SOAP_TYPE_json__i4
 # define SOAP_TYPE__int                 SOAP_TYPE_json__int
@@ -48,10 +49,13 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 # define SOAP_TYPE__struct              SOAP_TYPE_json__struct
 # define SOAP_TYPE__base64              SOAP_TYPE_json__base64
 #endif
+#endif
 
 #ifdef JSON_NAMESPACE
+#ifdef __cplusplus
 namespace json {
 #endif
+#endif
 
 /******************************************************************************\
  *
@@ -114,7 +118,7 @@ int json_send(struct soap *soap, const struct value *v)
     return SOAP_OK;
   switch (v->__type)
   {
-    case SOAP_TYPE__array: 
+    case SOAP_TYPE__array:
       if ((soap->mode & SOAP_XML_INDENT))
         n = 2 * (++soap->level % 40) + 2;
       if (soap_send_raw(soap, "["/*"]"*/, 1))
@@ -136,26 +140,26 @@ int json_send(struct soap *soap, const struct value *v)
         soap->level--;
       }
       return soap_send_raw(soap, /*"["*/"]", 1);
-    case SOAP_TYPE__boolean: 
+    case SOAP_TYPE__boolean:
       if (*(_boolean*)v->ref == 1)
         return soap_send_raw(soap, "true", 4);
       return soap_send_raw(soap, "false", 5);
-    case SOAP_TYPE__double: 
+    case SOAP_TYPE__double:
       return soap_send(soap, soap_double2s(soap, (double)*(_double*)v->ref));
-    case SOAP_TYPE__i4: 
+    case SOAP_TYPE__i4:
       return soap_send(soap, soap_int2s(soap, (int)*(_i4*)v->ref));
-    case SOAP_TYPE__int: 
+    case SOAP_TYPE__int:
       return soap_send(soap, soap_LONG642s(soap, (LONG64)*(_int*)v->ref));
-    case SOAP_TYPE__string: 
-    case SOAP_TYPE__dateTime_DOTiso8601: 
+    case SOAP_TYPE__string:
+    case SOAP_TYPE__dateTime_DOTiso8601:
       return json_send_string(soap, (const char*)v->ref);
-    case SOAP_TYPE__base64: 
+    case SOAP_TYPE__base64:
       if (soap_send_raw(soap, "\"", 1))
         return soap->error;
       if (v->ref && soap_putbase64(soap, ((struct _base64*)v->ref)->__ptr, ((struct _base64*)v->ref)->__size))
         return soap->error;
       return soap_send_raw(soap, "\"", 1);
-    case SOAP_TYPE__struct: 
+    case SOAP_TYPE__struct:
       if ((soap->mode & SOAP_XML_INDENT))
         n = 2 * (++soap->level % 40) + 2;
       if (soap_send_raw(soap, "{"/*"}"*/, 1))
@@ -328,7 +332,7 @@ int json_recv(struct soap *soap, struct value *v)
         if (json_recv(soap, &s))
           return soap->error;
         if (s.__type != SOAP_TYPE__string)
-          return soap_set_sender_error(soap, "field name expected", (const char*)s.ref, SOAP_SYNTAX_ERROR);
+          return soap_set_sender_error(soap, "string name expected", (const char*)s.ref, SOAP_SYNTAX_ERROR);
         while (((c = soap_getchar(soap)) > 0 && c <= 0x20) || c == 0xA0)
           continue;
         if (c != ':')
@@ -435,21 +439,25 @@ int json_recv(struct soap *soap, struct value *v)
                 {
                   case EOF:
                     return soap->error = SOAP_EOF;
+                  case '"':
+                  case '\\':
+                  case '/':
+                    break;
                   case 'b':
                     c = 8;
-                    break;               
+                    break;
                   case 'f':
                     c = 12;
-                    break;               
+                    break;
                   case 'n':
                     c = 10;
-                    break;               
+                    break;
                   case 'r':
                     c = 13;
-                    break;               
+                    break;
                   case 't':
                     c = 9;
-                    break;               
+                    break;
                   case 'u':
                   {
                     char *h;
@@ -463,13 +471,19 @@ int json_recv(struct soap *soap, struct value *v)
                         return soap->error = SOAP_EOF;
                       h[i] = c;
                     }
-                    wc[0] = soap_strtol(h, NULL, 16);
+                    h[4] = '\0';
+                    wc[0] = (wchar_t)soap_strtol(h, &h, 16);
                     wc[1] = 0;
+                    if (h - soap->tmpbuf < 4)
+                      return soap->error = SOAP_TYPE;
                     t = soap_wchar2s(soap, wc);
                     c = *t++;
                     if (!*t)
                       t = NULL;
+                    break;
                   }
+                  default:
+                    return soap_set_sender_error(soap, "invalid escape in string", NULL, SOAP_SYNTAX_ERROR);
                 }
                 *s++ = c;
                 l++;
@@ -492,8 +506,16 @@ int json_recv(struct soap *soap, struct value *v)
                   else
                     *s++ = '?';
                 }
+                /* the JSON "standard" does not permit ctrl chars in strings, we silently accept these
+                else if (c < 0x20 && c >= 0)
+                {
+                  return soap_set_sender_error(soap, "invalid control character in string", NULL, SOAP_SYNTAX_ERROR);
+                }
+                */
                 else
+                {
                   *s++ = c;
+                }
                 l++;
             }
           }
@@ -526,7 +548,7 @@ int json_recv(struct soap *soap, struct value *v)
         {
           double x;
           if (soap_s2double(soap, soap->tmpbuf, &x))
-            return soap_set_sender_error(soap, "number expected", soap->tmpbuf, SOAP_SYNTAX_ERROR);
+            return soap_set_sender_error(soap, "JSON number expected", soap->tmpbuf, SOAP_SYNTAX_ERROR);
           v->__type = SOAP_TYPE__double;
           if (!(v->ref = soap_malloc(soap, sizeof(_double))))
             return soap->error = SOAP_EOM;
@@ -549,7 +571,7 @@ int json_recv(struct soap *soap, struct value *v)
       }
       else if (strcmp(soap->tmpbuf, "null"))
       {
-        return soap_set_sender_error(soap, "value expected", soap->tmpbuf, SOAP_SYNTAX_ERROR);
+        return soap_set_sender_error(soap, "JSON value expected", soap->tmpbuf, SOAP_SYNTAX_ERROR);
       }
       return SOAP_OK;
     }
@@ -920,5 +942,7 @@ bool json_lne(const value& x, const value& y)
 #endif
 
 #ifdef JSON_NAMESPACE
+#ifdef __cplusplus
 } // namespace json
 #endif
+#endif
