@@ -47,7 +47,11 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 --------------------------------------------------------------------------------
 */
 
-#include "soapH.h"
+#ifdef JSON_NAMESPACE
+# include "jsonH.h"
+#else
+# include "soapH.h"
+#endif
 
 static int size2k(int n);
 
@@ -355,10 +359,12 @@ int nth_at(struct value *v, const char *s)
 
 int nth_atw(struct value *v, const wchar_t *s)
 {
+  const char *t;
+  int i;
   if (s == NULL)
     return -1;
-  const char *t = soap_wchar2s(NULL, s);
-  int i = nth_at(v, t);
+  t = soap_wchar2s(NULL, s);
+  i = nth_at(v, t);
   free((void*)t);
   return i;
 }
@@ -388,6 +394,11 @@ _boolean is_double(struct value *v)
   return v->__type == SOAP_TYPE__double;
 }
 
+_boolean is_number(struct value *v)
+{
+  return v->__type == SOAP_TYPE__i4 || v->__type == SOAP_TYPE__int || v->__type == SOAP_TYPE__double;
+}
+
 _boolean is_string(struct value *v)
 {
   return v->__type == SOAP_TYPE__string || (v->__any && *v->__any);
