@@ -243,7 +243,7 @@ value::operator wchar_t*() const
 {
   const char *s = *this;
   wchar_t *t = NULL;
-  soap_s2wchar(soap, s, &t, 0, -1, NULL);
+  soap_s2wchar(soap, s, &t, 1, 0, -1, NULL);
   soap->error = SOAP_OK;
   return t;
 }
@@ -252,7 +252,7 @@ value::operator std::wstring() const
 {
   const char *s = *this;
   wchar_t *t = NULL;
-  soap_s2wchar(soap, s, &t, 0, -1, NULL);
+  soap_s2wchar(soap, s, &t, 1, 0, -1, NULL);
   soap->error = SOAP_OK;
   return std::wstring(t);
 }
@@ -632,6 +632,11 @@ bool value::is_double() const
   return __type == SOAP_TYPE__double;
 }
 
+bool value::is_number() const
+{
+  return __type == SOAP_TYPE__i4 || __type == SOAP_TYPE__int || __type == SOAP_TYPE__double;
+}
+
 bool value::is_string() const
 {
   return __type == SOAP_TYPE__string || (__any && *__any);
