@@ -5,7 +5,7 @@
 
 --------------------------------------------------------------------------------
 gSOAP XML Web services tools
-Copyright (C) 2000-2015, Robert van Engelen, Genivia Inc. All Rights Reserved.
+Copyright (C) 2000-2017, Robert van Engelen, Genivia Inc. All Rights Reserved.
 This part of the software is released under one of the following licenses:
 GPL or Genivia's license for commercial use.
 --------------------------------------------------------------------------------
@@ -43,8 +43,8 @@ A commercial use license is available from Genivia, Inc., contact@genivia.com
 #include "error2.h"
 
 #ifndef VERSION
-# define VERSION "2.8.35" /* Current version */
-# define GSOAP_VERSION 20835
+# define VERSION "2.8.49" /* Current version */
+# define GSOAP_VERSION 20849
 #endif
 
 #ifdef WIN32
@@ -249,6 +249,7 @@ typedef	struct Tnode
 	Bool		incmax;
 	double		min;
 	double		max;
+        int             property;
 	const char	*pattern;
 } Tnode;
 
@@ -261,11 +262,14 @@ typedef	union Value {
 typedef	struct IDinfo {
 	Tnode		*typ;
 	Storage		sto;
-	Bool		hasval;		/* if true, identifier is constant */
+	Bool		hasval;		/* if true, identifier has a default value */
+	Bool		ptrval;		/* if true, identifier is a pointer to a default value */
+	Bool		fixed;		/* if true and hasval, identifier has a fixed value */
 	Value		val;		/* ... with this value */
 	int		offset;
 	LONG64		minOccurs;
 	LONG64		maxOccurs;
+	Bool		nillable;
 } IDinfo;
 
 typedef	struct Entry {
@@ -293,7 +297,8 @@ typedef struct FNinfo {
 typedef	struct Node {
 	Tnode		*typ;
 	Storage		sto;
-	Bool		hasval;		/* if true, this node has a constant value */
+	Bool		hasval;		/* if true, this node has a default value */
+	Bool		fixed;		/* if true and hasval, this node has a fixed value */
 	Value		val;		/* ... this is the value */
 	Bool		hasmin;
 	Bool		hasmax;
@@ -303,6 +308,8 @@ typedef	struct Node {
 	LONG64		maxOccurs;
 	double		min;
 	double		max;
+	Bool		nillable;
+        int             property;
 	const char	*pattern;
 } Node;
 
@@ -398,6 +405,8 @@ extern FILE *fmsg;
 
 extern int soap_version;
 
+extern const char *copt;
+
 extern int aflag;
 extern int Aflag;
 extern int bflag;
